@@ -19,6 +19,7 @@ impl Table {
         let raw_table = &mut self.raw_table;
         let tree_table = &mut self.tree_table;
         let refs_table = &mut self.refs_table;
+        let refs_to_table = &mut self.refs_to_table;
         let calculated_table = &mut self.calculated_table;
 
         if raw_table.len() <= y || raw_table[y].len() <= x {
@@ -26,20 +27,27 @@ impl Table {
         }
         raw_table[y][x] = raw_string.borrow().to_string();
 
-        for y_of_src in 0..refs_table.len() {
-            for x_or_src in 0..refs_table[y].len() {
-                refs_table[y_of_src][x_or_src].retain(|&v| v != (x, y));
+        for (x_of_src, y_of_src) in &refs_to_table[y][x] {
+            let x_of_src = *x_of_src;
+            let y_of_src = *y_of_src;
+            if y_of_src < refs_table.len() && x_of_src < refs_table[y].len() {
+                refs_table[y_of_src][x_of_src].retain(|&v| v != (x, y));
             }
         }
 
         let (tree, refs) = Self::build_tree(raw_string.borrow().to_string());
         tree_table[y][x] = tree;
+        refs_to_table[y][x] = refs;
 
-        for (x_of_src, y_of_src) in refs {
-            refs_table[y_of_src as usize][x_of_src as usize].push((x, y));
+        for &(x_of_src, y_of_src) in &refs_to_table[y][x] {
+            if y_of_src < refs_table.len() && x_of_src < refs_table[y].len() {
+                refs_table[y_of_src][x_of_src].push((x, y));
+            }
         }
 
-        Self::calc(x, y, &tree_table, &refs_table, calculated_table);
+        if !refs_table[y][x].contains(&(x, y)) {
+            Self::calc(x, y, &tree_table, &refs_table, calculated_table);
+        }
         Ok(())
     }
 }
